/* Public domain. */

#ifndef NEMO_CDB_H
#define NEMO_CDB_H

#include <nemo/stdint.h>
#include <nemo/stralloc.h>

#ifdef __cplusplus
extern "C" {
#endif

uint32_t	cdb_hash(const void *buf, unsigned int len);

struct cdb {
  int fd;
  char *map;		/* 0 if no map is available */
  uint32_t size;	/* initialized if map is nonzero */
  uint32_t loop;	/* number of hash slots searched under this key */
  uint32_t khash;	/* initialized if loop is nonzero */
  uint32_t kpos;	/* initialized if loop is nonzero */
  uint32_t hpos;	/* initialized if loop is nonzero */
  uint32_t hslots;	/* initialized if loop is nonzero */
  uint32_t dpos;	/* initialized if cdb_findnext() returns 1 */
  uint32_t dlen;	/* initialized if cdb_findnext() returns 1 */
};

#define	CDB {0, 0, 0, 0, 0, 0, 0, 0, 0, 0}

void		cdb_free(struct cdb *c);
void		cdb_init(struct cdb *c, int fd);

void		cdb_close(struct cdb *c);
unsigned int	cdb_open(struct cdb *c, const char *fn);

int		cdb_read(struct cdb *c, void *buf, unsigned int len, uint32_t pos);
int		cdb_read_stralloc(struct cdb *c, stralloc *sa);   /* append current result to sa */

void		cdb_findstart(struct cdb *c);
int		cdb_findnext(struct cdb *c, const void *key, unsigned int len);
int		cdb_find(struct cdb *c, const void *key, unsigned int len);

uint32_t	cdb_unpack(const void *buf);

static inline uint32_t	cdb_datapos(const struct cdb *c) { return c->dpos; }
static inline uint32_t	cdb_datalen(const struct cdb *c) { return c->dlen; }

#ifdef __cplusplus
}
#endif

#endif
