/* Public domain. */

#include <nemo/stdint.h>
#include <nemo/unixtypes.h>
#include <nemo/unix.h>
#include <nemo/byte.h>
#include <nemo/uint32.h>

#include "cdb.h"

void cdb_findstart(struct cdb *c)
{
  c->loop = 0;
}

static int match(struct cdb *c, const void *k, unsigned int len, uint32_t pos)
{
  register const byte_t *key;
  register unsigned int n;
  byte_t buf[32];
  key = k;
  while (len > 0) {
    n = sizeof buf;
    if (n > len) {
      n = len;
    }
    if (cdb_read(c, buf, n, pos) < 0) return -1;
    if (byte_diff(buf, n, key)) return 0;
    pos += n;
    key += n;
    len -= n;
  }
  return 1;
}

int cdb_findnext(register struct cdb *c, const void *key, unsigned int klen)
{
  byte_t buf[8];
  uint32_t pos;
  uint32_t u;

  if (!c->loop) {
    u = cdb_hash(key, klen);
    if (cdb_read(c, buf, sizeof(buf), (u << 3) & 2047) < 0) return -1;
    c->hslots = cdb_unpack(buf + 4);
    if (!c->hslots) return 0;
    c->hpos = cdb_unpack(buf);
    c->khash = u;
    u >>= 8;
    u %= c->hslots;
    u <<= 3;
    c->kpos = c->hpos + u;
  }
  while (c->loop < c->hslots) {
    if (cdb_read(c, buf, sizeof(buf), c->kpos) < 0) return -1;
    pos = cdb_unpack(buf + 4);
    if (!pos) return 0;
    c->loop += 1;
    c->kpos += 8;
    if (c->kpos == c->hpos + (c->hslots << 3)) {
      c->kpos = c->hpos;
    }
    u = cdb_unpack(buf);
    if (u == c->khash) {
      if (cdb_read(c, buf, sizeof(buf), pos) < 0) return -1;
      u = cdb_unpack(buf);
      if (u == klen) {
        switch (match(c, key, klen, pos + 8)) {
          case -1:
            return -1;
          case 1:
            c->dlen = cdb_unpack(buf + 4);
            c->dpos = pos + 8 + klen;
            return 1;
          default:
            break;
        }
      }
    }
  }
  return 0;
}

int cdb_find(struct cdb *c, const void *key, unsigned int klen)
{
  cdb_findstart(c);
  return cdb_findnext(c, key, klen);
}
