#include <nemo/fmt.h>
#include <nemo/stralloc.h>
#include <nemo/djbio.h>
#include <nemo/djbiofd.h>
#include <nemo/strerr.h>
#include <nemo/exit.h>
#include <nemo/stdint.h>
#include <nemo/uint32.h>
#include <nemo/macro_unused.h>

#include "cdb.h"

static const char FATAL[] =  "cdbdump: fatal: ";

static void die_write(void)
{
  strerr_die2sys(111, FATAL, "unable to write output");
}

static void put(const void *buf, unsigned int len)
{
  if (djbio_put(djbiofd_out, buf, len) < 0) die_write();
}
static void puteol(void)
{
  if (djbio_put(djbiofd_out, "\n", 1) < 0) die_write();
}
static void putflush(void)
{
  if (djbio_flush(djbiofd_out) < 0) die_write();
}

static uint32_t pos = 0;

static void get(void *out, unsigned int len)
{
  register int r;
  byte_t *buf;

  buf = out;
  while (len > 0) {
    r = djbio_get(djbiofd_in, buf, len);
    if (r < 0) {
      strerr_die2sys(111, FATAL, "unable to read input");
    }
    if (r == 0) {
      strerr_die2x(111, FATAL, "unable to read input: truncated file");
    }
    pos += (uint32_t)r;
    buf += r;
    len -= (unsigned int)r;
  }
}

static byte_t buf[512];

static void copy(uint32_t len)
{
  register unsigned int x;

  while (len) {
    x = sizeof buf;
    if (len < x) x = len;
    get(buf, x);
    put(buf, x);
    len -= x;
  }
}

static uint32_t getnum(void)
{
  get(buf, 4);
  return cdb_unpack(buf);
}

static char strnum[FMT_ULONG];

int main(int argc __UNUSED__, char **argv __UNUSED__)
{
  uint32_t eod;
  uint32_t klen;
  uint32_t dlen;

  eod = getnum();
  while (pos < 2048) {
    dlen = getnum();
  }
  while (pos < eod) {
    klen = getnum();
    dlen = getnum();

    put("+", 1);
    put(strnum, fmt_ulong(strnum, klen));

    put(", ", 1);
    put(strnum, fmt_ulong(strnum, dlen));

    put(":", 1);
    copy(klen);

    put("->", 2);
    copy(dlen);

    puteol();
  }
  puteol();
  putflush();
  _exit(0);
}
