#include <nemo/exit.h>
#include <nemo/scan.h>
#include <nemo/str.h>
#include <nemo/stralloc.h>
#include <nemo/djbio.h>
#include <nemo/djbiofd.h>
#include <nemo/strerr.h>

#include "cdb.h"

static const char FATAL[] = "cdbget: fatal: ";

static void die_read(void)
{
  strerr_die2sys(111, FATAL, "unable to read input");
}
static void die_write(void)
{
  strerr_die2sys(111, FATAL, "unable to write output");
}
static void die_usage(void)
{
  strerr_die1x(111, "cdbget: usage: cdbget key [skip]");
}

static struct cdb c;
static byte_t buf[1024];

int main(int argc, char **argv)
{
  char *key;
  int r;
  unsigned int s;
  uint32_t pos;
  uint32_t len;
  unsigned long u;

  if (argc < 2) die_usage();

  if (!*argv) die_usage();

  if (!*++argv) die_usage();
  key = *argv;

  u = 0;
  if (*++argv) {
    scan_ulong(*argv, &u);
  }

  cdb_init(&c, 0);
  cdb_findstart(&c);

  for (;;) {
    r = cdb_findnext(&c, key, str_len(key));
    if (r < 0) die_read();
    if (!r) _exit(100);
    if (!u) break;
    --u;
  }

  pos = cdb_datapos(&c);
  len = cdb_datalen(&c);

  while (len > 0) {
    s = sizeof buf;
    if (s > len) s = len;
    if (cdb_read(&c, buf, s, pos) < 0) die_read();
    if (djbio_put(djbiofd_outsmall, buf, s) < 0) die_write();
    pos += s;
    len -= s;
  }
  if (djbio_flush(djbiofd_outsmall) < 0) die_write();
  _exit (0);
}
