#include <nemo/fmt.h>
#include <nemo/stralloc.h>
#include <nemo/djbio.h>
#include <nemo/djbiofd.h>
#include <nemo/strerr.h>
#include <nemo/seek.h>
#include <nemo/str.h>
#include <nemo/exit.h>
#include <nemo/stdint.h>
#include <nemo/uint32.h>
#include <nemo/macro_unused.h>

#include "cdb.h"

static const char FATAL[] = "cdbstats: fatal: ";

static void die_read(void)
{
  strerr_die2sys(111, FATAL, "unable to read input");
}
static void die_readformat(void)
{
  strerr_die2x(111, FATAL, "unable to read input: truncated file");
}
static void die_write(void)
{
  strerr_die2sys(111, FATAL, "unable to write output");
}
static void put(const void *buf, unsigned int len)
{
  if (djbio_put(djbiofd_outsmall, buf, len) < 0) die_write();
}
static void putflush(void)
{
  if (djbio_flush(djbiofd_outsmall) < 0) die_write();
}

static uint32_t pos = 0;

static void get(void *out, unsigned int len)
{
  register int r;
  byte_t *buf;
  buf = out;
  while (len > 0) {
    r = djbio_get(djbiofd_in, buf, len);
    if (r < 0) die_read();
    if (r == 0) die_readformat();
    pos += (uint32_t)r;
    buf += r;
    len -= (unsigned int)r;
  }
}

static void getnum(uint32_t *num)
{
  byte_t buf[4];
  get(buf, 4);
  uint32_unpack(num, buf);
}

static char strnum[FMT_ULONG];

static void putnum(const char *label, unsigned long count)
{
  unsigned int i;
  put(label, str_len(label));
  for (i = fmt_ulong(0, count); i < 10; ++i) {
   put(" ", 1);
  }
  put(strnum, fmt_ulong(strnum, count));
  put("\n", 1);
}

static byte_t key[1024];

static struct cdb c;

static unsigned long numrecords;
static unsigned long numd[11];

int main(int argc __UNUSED__, char **argv __UNUSED__)
{
  uint32_t eod;
  uint32_t klen;
  uint32_t dlen;
  off_t rest;

  cdb_init(&c, 0);

  getnum(&eod);
  while (pos < 2048) {
    getnum(&dlen);
  }

  while (pos < eod) {
    getnum(&klen);
    getnum(&dlen);
    if (klen > sizeof key) {
      while (klen) {
        get(key, 1);
        --klen;
      }
    }
    else {
      get(key, klen);
      rest = seek_cur(0);
      cdb_findstart(&c);
      do {
        switch (cdb_findnext(&c, key, klen)) {
          case -1: die_read();
          case 0: die_readformat();
          default: break;
        }
      } while (cdb_datapos(&c) != pos);
      if (!c.loop) die_readformat();
      ++numrecords;
      if (c.loop > 10) {
        ++numd[10];
      }
      else {
        ++numd[c.loop - 1];
      }
      if (seek_set(0, rest) < 0) die_read();
    }
    while (dlen) {
      get(key, 1);
      --dlen;
    }
  }

  putnum("records ", numrecords);
  putnum("d0      ", numd[0]);
  putnum("d1      ", numd[1]);
  putnum("d2      ", numd[2]);
  putnum("d3      ", numd[3]);
  putnum("d4      ", numd[4]);
  putnum("d5      ", numd[5]);
  putnum("d6      ", numd[6]);
  putnum("d7      ", numd[7]);
  putnum("d8      ", numd[8]);
  putnum("d9      ", numd[9]);
  putnum(">9      ", numd[10]);
  putflush();
  _exit(0);
}
