#include <nemo/fmt.h>
#include <nemo/stralloc.h>
#include <nemo/djbio.h>
#include <nemo/djbiofd.h>
#include <nemo/strerr.h>
#include <nemo/seek.h>
#include <nemo/str.h>
#include <nemo/exit.h>
#include <nemo/stdint.h>
#include <nemo/uint32.h>
#include <nemo/macro_unused.h>

#include "cdb.h"

static const char FATAL[] = "cdbtest: fatal: ";

static void die_read(void)
{
  strerr_die2sys(111, FATAL, "unable to read input");
}
static void die_write(void)
{
  strerr_die2sys(111, FATAL, "unable to write output");
}
static void put(const void *buf, unsigned int len)
{
  if (djbio_put(djbiofd_outsmall, buf, len) < 0) die_write();
}
static void putflush(void)
{
  if (djbio_flush(djbiofd_outsmall) < 0) die_write();
}

static uint32_t pos = 0;

static void get(void *out, unsigned int len)
{
  register int r;
  byte_t *buf;
  buf = out;
  while (len > 0) {
    r = djbio_get(djbiofd_in, buf, len);
    if (r < 0) die_read();
    if (r == 0) {
      strerr_die2x(111, FATAL, "unable to read input: truncated file");
    }
    pos += (uint32_t)r;
    buf += r;
    len -= (unsigned int)r;
  }
}

static void getnum(uint32_t *num)
{
  byte_t buf[4];
  get(buf, 4);
  uint32_unpack(num, buf);
}

static char strnum[FMT_ULONG];

static void putnum(const char *label, unsigned long count)
{
  put(label, str_len(label));
  put(strnum, fmt_ulong(strnum, count));
  put("\n", 1);
}

static byte_t key[1024];

static unsigned long numuntested = 0;
static unsigned long numnotfound = 0;
static unsigned long numotherpos = 0;
static unsigned long numbadlen = 0;
static unsigned long numfound = 0;

static struct cdb c;

int main(int argc __UNUSED__, char **argv __UNUSED__)
{
  uint32_t eod;
  uint32_t klen;
  uint32_t dlen;
  off_t rest;

  cdb_init(&c, 0);

  getnum(&eod);
  while (pos < 2048) {
    getnum(&dlen);
  }

  while (pos < eod) {
    getnum(&klen);
    getnum(&dlen);
    if (klen > sizeof key) {
      ++numuntested;
      while (klen) {
        get(key, 1);
        --klen;
      }
    }
    else {
      get(key, klen);
      rest = seek_cur(0);
      switch (cdb_find(&c, key, klen)) {
        case -1:
          die_read();
        case 0:
          ++numnotfound;
          break;
        default:
          if (cdb_datapos(&c) != pos) {
            ++numotherpos;
          }
          else {
            if (cdb_datalen(&c) != dlen) {
              ++numbadlen;
            }
            else {
              ++numfound;
            }
          }
      }
      if (seek_set(0, rest) < 0) die_read();
    }
    while (dlen) {
      get(key, 1);
      --dlen;
    }
  }

  putnum("found: ", numfound);
  putnum("different record: ", numotherpos);
  putnum("bad length: ", numbadlen);
  putnum("not found: ", numnotfound);
  putnum("untested: ", numuntested);
  putflush();
  _exit(0);
}
